@file:OptIn(LowLevelApi::class)
@file:JvmName("_Row2RowKt")
@file:Suppress("DuplicatedCode")

package ru.landgrafhomyak.db.serdha0.user_commons.executors

import kotlin.jvm.JvmName
import ru.landgrafhomyak.db.serdha0.api.LowLevelApi
import ru.landgrafhomyak.db.serdha0.api.queries._Query
import ru.landgrafhomyak.db.serdha0.api.runtime.Transaction

public class ExpectedOneRowException : Error("Expected at least one row, but table is empty")
public class TooManyRowsException : Error("Expected exactly one row, but got more")

public suspend inline fun <qUE : Any, R> Transaction.selectExactlyOneOrError(
	compiledQuery: _Query.Params2Table<qUE>,
	params: RowProducer0<qUE>,
	result: RowConsumer0<qUE, R>
): R {
	val iRow = this._executeQuery(compiledQuery)
	_safeAutoClose(onAbort = { iRow._abort() }) {
		params.initializeRow(iRow)
	}
	val oRow = iRow._finish()
	val ret: R
	_safeAutoClose(onAbort = { oRow._abort() }) {
		if (!oRow._next())
			throw ExpectedOneRowException()
		ret = result.transformRow(oRow)
		if (oRow._next())
			throw TooManyRowsException()
	}
	oRow._finish()
	return ret
}

public suspend inline fun <qUE : Any, R> Transaction.selectExactlyOneOrError(
	compiledQuery: _Query.Void2Table<qUE>,
	result: RowConsumer0<qUE, R>
): R {
	val oRow = this._executeQuery(compiledQuery)
	val ret: R
	_safeAutoClose(onAbort = { oRow._abort() }) {
		if (!oRow._next())
			throw ExpectedOneRowException()
		ret = result.transformRow(oRow)
		if (oRow._next())
			throw TooManyRowsException()
	}
	oRow._finish()
	return ret
}

public suspend inline fun <qUE : Any, R> Transaction.selectExactlyOneOrNull(
	compiledQuery: _Query.Params2Table<qUE>,
	params: RowProducer0<qUE>,
	result: RowConsumer0<qUE, R>
): R? {
	val iRow = this._executeQuery(compiledQuery)
	_safeAutoClose(onAbort = { iRow._abort() }) {
		params.initializeRow(iRow)
	}
	val oRow = iRow._finish()
	val ret: R?
	_safeAutoClose(onAbort = { oRow._abort() }) {
		if (!oRow._next()) {
			ret = null
			return@_safeAutoClose
		}
		ret = result.transformRow(oRow)
		if (oRow._next())
			throw TooManyRowsException()
	}
	oRow._finish()
	return ret
}

public suspend inline fun <qUE : Any, R> Transaction.selectExactlyOneOrNull(
	compiledQuery: _Query.Void2Table<qUE>,
	result: RowConsumer0<qUE, R>
): R? {
	val oRow = this._executeQuery(compiledQuery)
	val ret: R?
	_safeAutoClose(onAbort = { oRow._abort() }) {
		if (!oRow._next()) {
			ret = null
			return@_safeAutoClose
		}
		ret = result.transformRow(oRow)
		if (oRow._next())
			throw TooManyRowsException()
	}
	oRow._finish()
	return ret
}
public suspend inline fun <qUE : Any, R> Transaction.selectFirstOrError(
	compiledQuery: _Query.Params2Table<qUE>,
	params: RowProducer0<qUE>,
	result: RowConsumer0<qUE, R>
): R {
	val iRow = this._executeQuery(compiledQuery)
	_safeAutoClose(onAbort = { iRow._abort() }) {
		params.initializeRow(iRow)
	}
	val oRow = iRow._finish()
	val ret: R
	_safeAutoClose(onAbort = { oRow._abort() }) {
		if (!oRow._next())
			throw ExpectedOneRowException()
		ret = result.transformRow(oRow)
	}
	oRow._finish()
	return ret
}

public suspend inline fun <qUE : Any, R> Transaction.selectFirstOrError(
	compiledQuery: _Query.Void2Table<qUE>,
	result: RowConsumer0<qUE, R>
): R {
	val oRow = this._executeQuery(compiledQuery)
	val ret: R
	_safeAutoClose(onAbort = { oRow._abort() }) {
		if (!oRow._next())
			throw ExpectedOneRowException()
		ret = result.transformRow(oRow)
	}
	oRow._finish()
	return ret
}

public suspend inline fun <qUE : Any, R> Transaction.selectFirstOrNull(
	compiledQuery: _Query.Params2Table<qUE>,
	params: RowProducer0<qUE>,
	result: RowConsumer0<qUE, R>
): R? {
	val iRow = this._executeQuery(compiledQuery)
	_safeAutoClose(onAbort = { iRow._abort() }) {
		params.initializeRow(iRow)
	}
	val oRow = iRow._finish()
	val ret: R?
	_safeAutoClose(onAbort = { oRow._abort() }) {
		if (!oRow._next()) {
			ret = null
			return@_safeAutoClose
		}
		ret = result.transformRow(oRow)
	}
	oRow._finish()
	return ret
}

public suspend inline fun <qUE : Any, R> Transaction.selectFirstOrNull(
	compiledQuery: _Query.Void2Table<qUE>,
	result: RowConsumer0<qUE, R>
): R? {
	val oRow = this._executeQuery(compiledQuery)
	val ret: R?
	_safeAutoClose(onAbort = { oRow._abort() }) {
		if (!oRow._next()) {
			ret = null
			return@_safeAutoClose
		}
		ret = result.transformRow(oRow)
	}
	oRow._finish()
	return ret
}