@file:OptIn(LowLevelApi::class)
@file:JvmName("_Rows2VoidKt")
@file:Suppress("DuplicatedCode")

package ru.landgrafhomyak.db.serdha0.user_commons.executors

import kotlin.contracts.contract
import kotlin.jvm.JvmName
import ru.landgrafhomyak.db.serdha0.api.LowLevelApi
import ru.landgrafhomyak.db.serdha0.api.queries._Query
import ru.landgrafhomyak.db.serdha0.api.runtime.Transaction

public suspend inline fun <qUE : Any, E> Transaction.mapToRows(
	compiledQuery: _Query.Table2Void<qUE>,
	collection: Iterable<E>,
	transform: RowProducer1<qUE, E>
) {
	val iRow = this._executeQuery(compiledQuery)
	_safeAutoClose(onAbort = { iRow._abort() }) {
		for (elem in collection) {
			iRow._next()
			transform.transformToRow(iRow, elem)
		}
	}
	iRow._finish()
}

public suspend inline fun <qUE : Any, E> Transaction.mapToRowsIndexed(
	compiledQuery: _Query.Table2Void<qUE>,
	collection: Iterable<E>,
	firstIndex: Int = 0,
	transform: RowProducer2<qUE, E>
) {
	val iRow = this._executeQuery(compiledQuery)
	var index = firstIndex
	_safeAutoClose(onAbort = { iRow._abort() }) {
		for (elem in collection) {
			iRow._next()
			transform.transformToRow(iRow, index, elem)
			index++
		}
	}
	iRow._finish()
}

public suspend inline fun <qUE : Any, E> Transaction.mapToRows(
	compiledQuery: _Query.Table2Void<qUE>,
	sequence: Sequence<E>,
	transform: RowProducer1<qUE, E>
) {
	val iRow = this._executeQuery(compiledQuery)
	_safeAutoClose(onAbort = { iRow._abort() }) {
		for (elem in sequence) {
			iRow._next()
			transform.transformToRow(iRow, elem)
		}
	}
	iRow._finish()
}

public suspend inline fun <qUE : Any, E> Transaction.mapToRowsIndexed(
	compiledQuery: _Query.Table2Void<qUE>,
	sequence: Sequence<E>,
	firstIndex: Int = 0,
	transform: RowProducer2<qUE, E>
) {
	val iRow = this._executeQuery(compiledQuery)
	var index = firstIndex
	_safeAutoClose(onAbort = { iRow._abort() }) {
		for (elem in sequence) {
			iRow._next()
			transform.transformToRow(iRow, index, elem)
			index++
		}
	}
	iRow._finish()
}

public suspend inline fun <qUE : Any, E> Transaction.mapToRows(
	compiledQuery: _Query.Table2Void<qUE>,
	sequence: Array<E>,
	transform: RowProducer1<qUE, E>
) {
	val iRow = this._executeQuery(compiledQuery)
	_safeAutoClose(onAbort = { iRow._abort() }) {
		for (elem in sequence) {
			iRow._next()
			transform.transformToRow(iRow, elem)
		}
	}
	iRow._finish()
}

public suspend inline fun <qUE : Any, E> Transaction.mapToRowsIndexed(
	compiledQuery: _Query.Table2Void<qUE>,
	sequence: Array<E>,
	firstIndex: Int = 0,
	transform: RowProducer2<qUE, E>
) {
	val iRow = this._executeQuery(compiledQuery)
	var index = firstIndex
	_safeAutoClose(onAbort = { iRow._abort() }) {
		for (elem in sequence) {
			iRow._next()
			transform.transformToRow(iRow, index, elem)
			index++
		}
	}
	iRow._finish()
}